'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));
var reactRedux = require('react-redux');

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var get = function get(object, property, receiver) {
  if (object === null) object = Function.prototype;
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent === null) {
      return undefined;
    } else {
      return get(parent, property, receiver);
    }
  } else if ("value" in desc) {
    return desc.value;
  } else {
    var getter = desc.get;

    if (getter === undefined) {
      return undefined;
    }

    return getter.call(receiver);
  }
};

















var set = function set(object, property, value, receiver) {
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent !== null) {
      set(parent, property, value, receiver);
    }
  } else if ("value" in desc && desc.writable) {
    desc.value = value;
  } else {
    var setter = desc.set;

    if (setter !== undefined) {
      setter.call(receiver, value);
    }
  }

  return value;
};

function chooseRandomly(variants) {
	var weighedVariants = [];
	variants.forEach(function (variant) {
		var weight = variant.weight || 1;
		for (var i = 0; i < weight; i++) {
			weighedVariants.push(variant);
		}
	});
	return weighedVariants[Math.floor(Math.random() * weighedVariants.length)].name;
}

function getRandomVariant(experiment) {
	var choose = experiment.choose || chooseRandomly;
	return choose(experiment.variants);
}

function createExperiments(experiments) {
	function generateRandomState() {
		var initialState = {};
		for (var name in experiments) {
			initialState[name] = getRandomVariant(experiments[name]);
		}
		return initialState;
	}
	return function reducer() {
		var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : generateRandomState();
		var action = arguments[1];

		switch (action.type) {
			case 'RESET_EXPERIMENT_VARIANT':
				return Object.assign({}, state, defineProperty({}, action.experiment, getRandomVariant(experiments[action.experiment])));

			case 'LOAD_EXPERIMENTS_VARIANTS':
				{
					var newState = {};
					for (var key in state) {
						if (action.state.hasOwnProperty(key) && state.hasOwnProperty(key)) {
							newState[key] = action.state[key];
						} else {
							newState[key] = state[key];
						}
					}
					return newState;
				}

			case 'SET_EXPERIMENT_VARIANT':
				return Object.assign({}, state, defineProperty({}, action.experiment, action.variant));

			default:
				return state;
		}
	};
}

function Variant(props) {
	return React.Children.only(props.children);
}

Variant.propTypes = {
	name: PropTypes.string.isRequired
};

function setExperimentVariant(experiment, variant) {
	return {
		type: 'SET_EXPERIMENT_VARIANT',
		experiment: experiment,
		variant: variant
	};
}

function loadExperimentVariants(state) {
	return {
		type: 'LOAD_EXPERIMENTS_VARIANTS',
		state: state
	};
}

function Selector(props) {
	var variant = props.variant,
	    name = props.name,
	    children = props.children;

	var chosenOne = void 0;
	React.Children.forEach(children, function (child) {
		if (child.type === Variant && child.props.name === variant) {
			chosenOne = React.cloneElement(child, { experiment: name });
		}
	});
	return chosenOne || null;
}

Selector.propTypes = {
	name: PropTypes.string.isRequired
};

var mapStateToProps = function mapStateToProps(state, ownProps) {
	return {
		variant: state.experiments[ownProps.name] || null
	};
};

var Experiment = reactRedux.connect(mapStateToProps)(Selector);

var prefix = '_react_redux_ab_';

function digestCookies(cookies) {
	var experiments = {};
	for (var key in cookies) {
		if (key.indexOf(prefix) === 0) {
			var name = key.slice(prefix.length);
			experiments[name] = cookies[key];
		}
	}
	return experiments;
}

function bakeCookies(state, setter) {
	for (var experiment in state.experiments) {
		setter(prefix + experiment, state.experiments[experiment]);
	}
}

exports.createExperiments = createExperiments;
exports.Variant = Variant;
exports.Experiment = Experiment;
exports.setExperimentVariant = setExperimentVariant;
exports.loadExperimentVariants = loadExperimentVariants;
exports.digestCookies = digestCookies;
exports.bakeCookies = bakeCookies;
